/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.model;

import java.awt.Image;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;
import org.openide.util.ImageUtilities;

/**
 *
 * @author Maciek Dulkiewicz && Wytse Visser
 */
public class Implement extends Machine {

    private transient ImplementGeographical implementGeographical;
    public static final String EXT = "imt";
    public static final ItemIdType ITEM_ID_TYPE = ItemIdType.IMT;
    private double workWidth = 0.0d;
    public static final String PROP_WORKWIDTH = "workWidth";

    public Implement(EnterpriseIdProvider eip, String name) {
        super(eip, ITEM_ID_TYPE, name);
    }

    public double getWorkWidth() {
        return workWidth;
    }

    public void setWorkWidth(double workWidth) {
        double old = this.workWidth;
        this.workWidth = workWidth;
        getPropertyChangeSupport().firePropertyChange(PROP_WORKWIDTH, old, workWidth);
    }

    @Override
    public MachineGeometrical getGeometrical() {
        if (implementGeographical == null) {
            implementGeographical = new ImplementGeographical();
        }
        return implementGeographical;
    }

    private class ImplementGeographical extends MachineGeometrical {

        public ImplementGeographical() {
            super(Implement.this);
        }

        @Override
        public Image getIcon() {
            return ImageUtilities.loadImage("nl/cloudfarming/client/fleet/implement/widgets32.png");
        }
    }
}
