/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.model;

import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import javax.swing.ImageIcon;
import nl.cloudfarming.client.model.AbstractEntity;
import nl.cloudfarming.client.model.AgroURI;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;
import org.openide.nodes.Node.Property;
import org.openide.nodes.PropertySupport;

/**
 *
 * @author Wytse Visser
 */
public abstract class Machine extends AbstractEntity implements Serializable {

//    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.model.Machine");
    private String name, type, brand, modelNumber;
    private Date purchaseDate;
    private double longitude, latitude;
    private int purchasePrice, mileage, year;
    private Set<MachineProperty> brandProperties;
    private ImageIcon image;
    public static final String PROP_NAME = "name";
    public static final String PROP_TYPE = "type";
    public static final String PROP_BRAND = "brand";
    public static final String PROP_MODELNUMBER = "modelNumber";
    public static final String PROP_YEAR = "year";
    public static final String PROP_PURCHASEDATE = "purchaseDate";
    public static final String PROP_PURCHASEPRICE = "purchasePrice";
    public static final String PROP_MILEAGE = "mileage";

    /**
     * Returns geometrical representation of this machine
     *
     * @return
     */
    public abstract MachineGeometrical getGeometrical();

    protected Machine(AgroURI uri) {
        super(uri);
    }

    protected Machine(EnterpriseIdProvider eip, ItemIdType idType, String name) {
        super(eip, idType);
        this.name = name;

        // Some default values for development purposes
        longitude = 3.873;
        latitude = 51.538;

        brandProperties = new HashSet<>();
    }

    /**
     * Get the value of name
     *
     * @return the value of name
     */
    public String getName() {
        return name;
    }

    /**
     * Set the value of name
     *
     * @param name new value of name
     */
    public void setName(String name) {
        String oldName = this.name;
        this.name = name;
        getPropertyChangeSupport().firePropertyChange(PROP_NAME, oldName, name);
    }

    /**
     * Get the value of type
     *
     * @return the value of type
     */
    public String getType() {
        return type;
    }

    /**
     * Set the value of type
     *
     * @param type new value of type
     */
    public void setType(String type) {
        String oldType = this.type;
        this.type = type;
        getPropertyChangeSupport().firePropertyChange(PROP_TYPE, oldType, type);
    }

    /**
     * Get the value of brand
     *
     * @return the value of brand
     */
    public String getBrand() {
        return brand;
    }

    /**
     * Set the value of brand
     *
     * @param brand new value of brand
     */
    public void setBrand(String brand) {
        String oldBrand = this.brand;
        this.brand = brand;
        getPropertyChangeSupport().firePropertyChange(PROP_BRAND, oldBrand, brand);
    }

    /**
     * Get the value of model
     *
     * @return the value of model
     */
    public String getModelNumber() {
        return modelNumber;
    }

    /**
     * Set the value of model
     *
     * @param model new value of model
     */
    public void setModelNumber(String model) {
        String oldModel = this.modelNumber;
        this.modelNumber = model;
        getPropertyChangeSupport().firePropertyChange(PROP_MODELNUMBER, oldModel, model);
    }

    /**
     * Get the value of year
     *
     * @return the value of year
     */
    public int getYear() {
        return year;
    }

    /**
     * Set the value of year
     *
     * @param year new value of year
     */
    public void setYear(int year) {
        int oldYear = this.year;
        this.year = year;
        getPropertyChangeSupport().firePropertyChange(PROP_YEAR, oldYear, year);
    }

    /**
     * Get the value of purchaseDate
     *
     * @return the value of purchaseDate
     */
    public Date getPurchaseDate() {
        return purchaseDate;
    }

    /**
     * Set the value of purchaseDate
     *
     * @param purchaseDate new value of purchaseDate
     */
    public void setPurchaseDate(Date purchaseDate) {
        Date oldPurchaseDate = this.purchaseDate;
        this.purchaseDate = purchaseDate;
        getPropertyChangeSupport().firePropertyChange(PROP_PURCHASEDATE, oldPurchaseDate, purchaseDate);
    }

    /**
     * Get the value of purchasePrice
     *
     * @return the value of purchasePrice
     */
    public int getPurchasePrice() {
        return purchasePrice;
    }

    /**
     * Set the value of purchasePrice
     *
     * @param purchasePrice new value of purchasePrice
     */
    public void setPurchasePrice(int purchasePrice) {
        double oldPurchasePrice = this.purchasePrice;
        this.purchasePrice = purchasePrice;
        getPropertyChangeSupport().firePropertyChange(PROP_PURCHASEPRICE, oldPurchasePrice, purchasePrice);
    }

    /**
     * Get the value of mileage
     *
     * @return the value of mileage
     */
    public int getMileage() {
        return mileage;
    }

    /**
     * Set the value of mileage
     *
     * @param mileage new value of mileage
     */
    public void setMileage(int mileage) {
        int oldMileage = this.mileage;
        this.mileage = mileage;
        getPropertyChangeSupport().firePropertyChange(PROP_MILEAGE, oldMileage, mileage);
    }

    public Double getLongitude() {
        return longitude;
    }

    public void setLongitude(Double longitude) {
        double oldLongitude = this.longitude;
        this.longitude = longitude;
        getPropertyChangeSupport().firePropertyChange(PROP_LONGITUDE, oldLongitude, longitude);
    }
    public static final String PROP_LONGITUDE = "longitude";

    public Double getLatitude() {
        return latitude;
    }

    public void setLatitude(Double latitude) {
        double oldLatitude = this.latitude;
        this.latitude = latitude;
        getPropertyChangeSupport().firePropertyChange(PROP_LATITUDE, oldLatitude, latitude);
    }
    public static final String PROP_LATITUDE = "latitude";

    public void setPosition(Double longitude, Double latitude) {
        Double[] oldPosition = new Double[]{this.latitude, this.longitude};
        this.latitude = latitude;
        this.longitude = longitude;
        Double[] newPosition = new Double[]{this.latitude, this.longitude};
        getPropertyChangeSupport().firePropertyChange(PROP_POSITION, oldPosition, newPosition);
    }
    public static final String PROP_POSITION = "position";

    public ImageIcon getImage() {
        return image;
    }

    public void setImage(ImageIcon image) {
        ImageIcon oldImage = this.image;
        this.image = image;
        getPropertyChangeSupport().firePropertyChange(PROP_IMAGE, oldImage, image);
    }
    public static final String PROP_IMAGE = "image";

    public void addProperty(MachineProperty property) {
        brandProperties.add(property);
    }

    protected class ReflectionProperty extends PropertySupport.Reflection {

        public ReflectionProperty(Class valueType, String property, String displayName) throws NoSuchMethodException {
            super(Machine.this, valueType, property);
            setDisplayName(displayName);
        }
    }

    public interface MachineMetaInfo {

        public String getDisplayName();

        public String getSmallIconResource();

        public String getLargeIconResource();
    }
}