/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.model.gps;

import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import nl.cloudfarming.client.fleet.model.Machine;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * This action creates a GPS tracker for its machine and provides the ability to
 * enable and disable it.
 *
 * @author Wytse Visser
 */
@NbBundle.Messages({"enable_gps_tracking=Enable gps tracking", "disable_gps_tracking=Disable gps tracking"})
public class GPSAction extends AbstractAction {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.model.gps.GPSAction");
    private Machine machine;
    private GPSTracker tracker;
    private static Map<Machine, GPSAction> actionMap;

    protected GPSAction(Machine machine) {
        this.machine = machine;
        updateName();
    }

    /**
     * Get the instance of the GPSAction for the given machine.
     *
     * @param machine The machine
     * @return The instance of the GPSAction for the given machine
     */
    public static Action get(Machine machine) {
        GPSAction action = getActionMap().get(machine);
        if (action == null) {
            action = new GPSAction(machine);
            actionMap.put(machine, action);
        }
        action.setEnabled(action.getGPSTracker() != null);
        return action;
    }

    private static Map<Machine, GPSAction> getActionMap() {
        if (actionMap == null) {
            actionMap = new HashMap<>();
        }
        return actionMap;
    }

    protected GPSTracker getGPSTracker() {
        if (tracker == null) {
            TrackerFactory factory = Lookup.getDefault().lookup(TrackerFactory.class);
            if (factory != null) {
                tracker = factory.createTracker();
                tracker.setMachine(machine);
            } else {
                LOGGER.warning("No GPSTracker installed.");
            }
        }
        return tracker;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        if (getGPSTracker() != null) {
            LOGGER.finest("GPSTRACKER != null");
            getGPSTracker().toggle();
            updateName();
        } else {
            LOGGER.finest("GPSTRACKER == null");
        }
    }

    private void updateName() {
        if (getGPSTracker() != null && getGPSTracker().isEnabled()) {
            putValue(NAME, Bundle.disable_gps_tracking());
        } else {
            putValue(NAME, Bundle.enable_gps_tracking());
        }
    }
}
